import * as _feathersjs_feathers from '@feathersjs/feathers';
import { HookContext, Params, Id, NextFunction, Paginated } from '@feathersjs/feathers';

type Promisable<T> = T | PromiseLike<T>;

type Change<T = any> = {
    before: T;
    item: T;
};
type ChangesById<T = any> = {
    [key: string]: Change<T>;
    [key: number]: Change<T>;
};
type ManipulateParams<H extends HookContext = HookContext> = (params: Params, context: H) => Promisable<Params | null>;
interface HookChangesByIdOptions<H extends HookContext = HookContext> {
    skipHooks: boolean;
    params?: ManipulateParams<H>;
    deleteParams?: string[];
    name?: string | string[];
    /** @default false */
    fetchBefore?: boolean;
}
interface ChangesByIdParams extends Params {
    changesById: any;
}
declare module "@feathersjs/feathers" {
    interface Params {
        paginate?: any;
        changesById?: any;
    }
}
declare const changesById: <H extends HookContext, T = any>(cb: (changesById: Record<Id, Change<T>>, context: H) => void | Promise<void>, _options?: Partial<HookChangesByIdOptions<H>>) => (context: H, next?: NextFunction) => Promise<H>;
declare const changesByIdBefore: <H extends HookContext>(context: H, _options: HookChangesByIdOptions<H>) => Promise<Record<string, unknown> | unknown[]>;
declare const changesByIdAfter: <H extends HookContext, T = any>(context: H, itemsBefore: any, cb?: ((changesById: Record<Id, Change<T>>, context: H) => void | Promise<void>) | null, _options?: HookChangesByIdOptions<H>) => Promise<Record<Id, Change> | undefined>;
type GetOrFindByIdParamsOptions<H extends HookContext = HookContext> = Pick<HookChangesByIdOptions<H>, "params" | "skipHooks" | "deleteParams"> & {
    type: "before" | "after";
};
declare const getOrFindByIdParams: <H extends HookContext = HookContext<_feathersjs_feathers.Application<any, any>, any>>(context: H, options: GetOrFindByIdParamsOptions<H>) => Promise<Params | undefined>;
type GetOrFindByIdOptions<H extends HookContext = HookContext> = GetOrFindByIdParamsOptions<H> & {
    byId?: boolean;
};

interface ViewContext<H extends HookContext = HookContext, T = any> {
    item: Change<T>;
    subscription: Subscription<H, T>;
    subscriptions: Subscription<H, T>[];
    items: Change<T>[];
    context: HookContext;
}
type ActionOptions<H extends HookContext = HookContext, T = any> = {
    subscription: SubscriptionResolved<H, T>;
    items: Change<T>[];
    context: H;
    view: Record<string, any>;
};
type Action<H extends HookContext = HookContext, T = any> = (change: Change<T>, options: ActionOptions<H, T>) => Promisable<void>;
type BatchAction<H extends HookContext = HookContext, T = any> = (changes: [change: Change<T>, options: ActionOptions<H, T>][], context: H) => Promisable<void>;
type HookTriggerOptions<H extends HookContext = HookContext, T = any> = Subscription<H, T> | (Subscription<H, T>[] & {
    batchMode?: never;
}) | (((context: H) => Promisable<Subscription<H, T> | Subscription<H, T>[]>) & {
    batchMode?: never;
});
type TransformView<H extends HookContext = HookContext, T = any> = undefined | ((view: Record<string, any>, viewContext: ViewContext<H, T>) => Promisable<Record<string, any>>) | Record<string, any>;
type Condition<H extends HookContext, T = Record<string, any>> = true | Record<string, any> | ((item: T, context: H) => Promisable<boolean>);
interface SubscriptionBase<H extends HookContext = HookContext, T = Record<string, any>> {
    /**
     * The name of the subscription
     *
     * Can be used to filter subscriptions
     */
    name?: string;
    service?: string | string[];
    method?: string | string[];
    conditionsData?: Condition<H, T>;
    conditionsResult?: Condition<H, T>;
    conditionsBefore?: Condition<H, T>;
    conditionsParams?: Condition<H, T>;
    view?: TransformView<H, T>;
    params?: ManipulateParams;
    /** @default true */
    isBlocking?: boolean;
    /** @default false */
    fetchBefore?: boolean;
    [key: string]: any;
    [key: number]: any;
}
type SubscriptionStandardMode<H extends HookContext, T> = {
    batchMode?: false;
    action: Action<H, T>;
} & SubscriptionBase<H, T>;
type SubscriptionBatchMode<H extends HookContext, T> = {
    batchMode: true;
    action: BatchAction<H, T>;
} & SubscriptionBase<H, T>;
type Subscription<H extends HookContext = HookContext, T = Record<string, any>> = SubscriptionStandardMode<H, T> | SubscriptionBatchMode<H, T>;
type SubscriptionResolvedBase<H extends HookContext = HookContext, T = Record<string, any>> = {
    dataResolved: boolean | Record<string, any>;
    resultResolved: boolean | Record<string, any>;
    beforeResolved: boolean | Record<string, any>;
    paramsResolved: Params;
    identifier: string;
};
type SubscriptionResolvedStandardMode<H extends HookContext = HookContext, T = Record<string, any>> = SubscriptionResolvedBase<H, T> & SubscriptionStandardMode<H, T>;
type SubscriptionResolvedBatchMode<H extends HookContext = HookContext, T = Record<string, any>> = SubscriptionResolvedBase<H, T> & SubscriptionBatchMode<H, T>;
type SubscriptionResolved<H extends HookContext = HookContext, T = Record<string, any>> = SubscriptionResolvedStandardMode<H, T> | SubscriptionResolvedBatchMode<H, T>;
declare const trigger: <H extends HookContext, T = H extends HookContext<infer app, infer S> ? S extends Partial<_feathersjs_feathers.ServiceMethods<infer TT, Partial<infer TT>, Params<_feathersjs_feathers.Query>, Partial<Partial<infer TT>>>> ? TT extends Paginated<infer TTT> ? TTT : TT extends (infer TTT_1)[] ? TTT_1 : TT : any : any, Options extends HookTriggerOptions<H, T> = HookTriggerOptions<H, T>>(options: Options) => (context: H, next?: NextFunction) => Promise<H>;

export { type Action, type ActionOptions, type BatchAction, type Change, type ChangesById, type ChangesByIdParams, type Condition, type GetOrFindByIdOptions, type GetOrFindByIdParamsOptions, type HookChangesByIdOptions, type HookTriggerOptions, type ManipulateParams, type Subscription, type SubscriptionBase, type SubscriptionBatchMode, type SubscriptionResolved, type SubscriptionResolvedBase, type SubscriptionResolvedBatchMode, type SubscriptionResolvedStandardMode, type SubscriptionStandardMode, type TransformView, changesById, changesByIdAfter, changesByIdBefore, getOrFindByIdParams, trigger };
